/*
 * Copyright (c) 2013-2020 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Controls 1.0

Rectangle {
    property real timeScale: 1.0
    property int adjustment: 0
    property real intervalSeconds: ((timeScale > 5)? 1 : (5 * Math.max(1, Math.floor(1.5 / timeScale)))) + adjustment

    SystemPalette { id: activePalette }

    id: rulerTop
    height: 28
    color: activePalette.base

    Repeater {
        model: parent.width / (intervalSeconds * profile.fps * timeScale)
        Rectangle {
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 1
            color: activePalette.windowText
            x: index * intervalSeconds * profile.fps * timeScale
            visible: ((x + width)   > scrollView.flickableItem.contentX) && // right edge
                      (x            < scrollView.flickableItem.contentX + scrollView.width) && // left edge
                     ((y + height)  > scrollView.flickableItem.contentY) && // bottom edge
                      (y            < scrollView.flickableItem.contentY + scrollView.height) // top edge

            Label {
                anchors.left: parent.right
                anchors.leftMargin: 2
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 2
                color: activePalette.windowText
                //Suppression d'un digit sur l'heure et réduction de la police
                text: application.timecode(index * intervalSeconds * profile.fps + 2).substr(1, 7)
                font.pointSize: font.pointSize - 2
            }
        }
    }
    Rectangle {
            visible: toolbar.colorMark.visible
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 3
            color: '#FF0090FF'
            x: toolbar.mark1 * timeScale - 1
    }
    Rectangle {
            visible: toolbar.colorMark.visible
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 3
            color: '#FFFFFF00'
            x: toolbar.mark2 * timeScale - 1
    }
    Rectangle {
            visible: toolbar.colorMark.visible
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 3
            color: '#FFFF00FF'
            x: toolbar.mark3 * timeScale - 1
    }
    Rectangle {
            visible: toolbar.colorMark.visible
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 3
            color: '#FFFF6600'
            x: toolbar.mark4 * timeScale - 1
    }
    Rectangle {
            visible: toolbar.colorMark.visible
            anchors.bottom: rulerTop.bottom
            height: 18
            width: 3
            color: '#FF66FFFF'
            x: toolbar.mark5 * timeScale - 1
    }

    MouseArea {
        anchors.fill: parent
        hoverEnabled: true
        acceptedButtons: Qt.NoButton
        onExited: bubbleHelp.hide()
        onPositionChanged: {
            //Suppression d'un digit sur l'heure dans la bulle
            var text = application.timecode(mouse.x / timeScale).substr(1, 10)
            bubbleHelp.show(mouse.x + bubbleHelp.width - 8, mouse.y + 65, text)
        }
    }

    Connections {
        target: profile
        onProfileChanged: {
            // Force a repeater model change to update the labels.
            ++adjustment
            --adjustment
        }
    }
}
