/*
 * Copyright (c) 2013-2020 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.2
import QtQuick.Controls 1.3
import QtQuick.Layouts 1.0
import QtGraphicalEffects 1.0
import Shotcut.Controls 1.0 as Shotcut
import QtQuick.Dialogs 1.1
import QtQuick.Controls.Styles 1.2 // Ajouté pour permettre la modif du style des contrôles
import Shotcut.Controls 1.0 // Ajouté pour permettre la bulle d'aide du combobox

ToolBar {
    property alias scrub: scrubButton.checked
    property color checkedColor: Qt.rgba(activePalette.highlight.r, activePalette.highlight.g, activePalette.highlight.b, 0.4)
    property alias scaleSlider: scaleSlider
    property alias mark1: m1.text
    property alias mark2: m2.text
    property alias mark3: m3.text
    property alias mark4: m4.text
    property alias mark5: m5.text
    property alias colorMark: colorMark // Ajouté pour piloter depuis le menu dans timeline.qml
    property alias deplacement: deplacement
    property int selectedmark: 0

    SystemPalette { id: activePalette }

    id: toolbar // Ajouté pour piloter depuis le menu dans timeline.qml
    width: 200
    height: settings.smallIcons? 28 : snapButton.height + 4
    anchors.margins: 0
    
//*** Ajout fonction pour Zoom (Playhead ne sort pas de l'écran) **********    
    function scrollIfNeeded() {
    	var x = timeline.position * multitrack.scaleFactor
    	if (x < scrollView.width * 0.1)
            scrollView.flickableItem.contentX = 0
        else
        	scrollView.flickableItem.contentX = x - scrollView.width * 0.5
	}

    function isMarker() {
        if (timeline.position == m1.text || timeline.position == m2.text || timeline.position == m3.text || timeline.position == m4.text || timeline.position == m5.text) return true
        else return false
    }

    function isMarkerFull() {
        if (m1.text !== '0' && m2.text !== '0' && m3.text !== '0' && m4.text !== '0' && m5.text !== '0') return true
    }

    RowLayout {
        ToolButton {
            id: hiddenButton
            visible: false
        }
        ToolButton {
            action: menuAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        Button { // separator
            enabled: false
            implicitWidth: 5
            implicitHeight: settings.smallIcons? 14 : (hiddenButton.implicitHeight - 8)
        }
        ToolButton {
            action: cutAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: copyAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: insertAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        Button { // separator
            enabled: false
            implicitWidth: 5
            implicitHeight: settings.smallIcons? 14 : (hiddenButton.implicitHeight - 8)
        }
        ToolButton {
            action: appendAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: deleteAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: liftAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: overwriteAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: splitAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        Button { // separator
            enabled: false
            implicitWidth: 5
            implicitHeight: settings.smallIcons? 14 : (hiddenButton.implicitHeight - 8)
        }
        Shotcut.ToolBarToggle {
            id: snapButton
            checked: settings.timelineSnap
            iconName: 'snap'
            iconSource: 'qrc:///icons/oxygen/32x32/actions/snap.png'
            tooltip: qsTr('Toggle snapping')
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            onClicked: settings.timelineSnap = !settings.timelineSnap
        }
        Shotcut.ToolBarToggle {
            id: scrubButton
            checked: settings.timelineDragScrub
            iconName: 'scrub_drag'
            iconSource: 'qrc:///icons/oxygen/32x32/actions/scrub_drag.png'
            tooltip: qsTr('Scrub while dragging')
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            onClicked: settings.timelineDragScrub = !settings.timelineDragScrub
        }
        Shotcut.ToolBarToggle {
            id: rippleButton
            checked: settings.timelineRipple
            iconName: 'target'
            iconSource: 'qrc:///icons/oxygen/32x32/actions/target.png'
            tooltip: qsTr('Ripple trim and drop')
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            onClicked: settings.timelineRipple = !settings.timelineRipple
        }
        Shotcut.ToolBarToggle {
            id: rippleAllButton
            checked: settings.timelineRippleAllTracks
            iconName: 'ripple-all'
            iconSource: 'qrc:///icons/oxygen/32x32/actions/ripple-all.png'
            tooltip: qsTr('Ripple edits across all tracks')
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            onClicked: settings.timelineRippleAllTracks = !settings.timelineRippleAllTracks
        }
        Button { // separator
            enabled: false
            visible: deplacement.visible
            implicitWidth: 5
            implicitHeight: settings.smallIcons? 14 : (hiddenButton.implicitHeight - 8)
        }
        // ********** début modif déplacement playhead **********************
        ToolButton {
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            visible: deplacement.visible
            action: Action {
                tooltip: qsTr('Reculer la tête de ' + parseInt(deplacement.text) + ' images.')
                iconName: 'media-seek-backward'
                iconSource: 'qrc:///icons/oxygen/32x32/actions/media-seek-backward.png'
                onTriggered: timeline.position = timeline.position - parseInt(deplacement.text)
            }
        }
        TextField {
            id: deplacement
            visible: false
            font.family: 'Verdana'
            font.pointSize: (application.OS === 'OS X')? 11 : 8
            implicitWidth: 35
            horizontalAlignment: TextInput.AlignHCenter
            text: '10'
            Keys.onEnterPressed: focus = false//hiddenButton.forceActiveFocus()
        }
        ToolButton {
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
            visible: deplacement.visible
            action: Action {
                tooltip: qsTr('Avancer la tête de ' + parseInt(deplacement.text) + ' images.')
                iconName: 'media-seek-forward'
                iconSource: 'qrc:///icons/oxygen/32x32/actions/media-seek-forward.png.png'
                onTriggered: timeline.position = timeline.position + parseInt(deplacement.text)
            }
        }
        Button { // separator
            visible: colorMark.visible
            enabled: false
            implicitWidth: 5
            implicitHeight: 18
        }
        // ********** fin modif déplacement playhead ************************
        // *********** Modif ajout Marqueurs et Compteur partiel *************
        TextField {
            id: origine
            text: "0"
            visible: false
        }
        TextField {
            id: m1
            text: "0"
            visible: false
        }
        TextField {
            id: m2
            text: "0"
            visible: false
        }
        TextField {
            id: m3
            text: "0"
            visible: false
        }
        TextField {
            id: m4
            text: "0"
            visible: false
        }
        TextField {
            id: m5
            text: "0"
            visible: false
        }
        ToolButton {
            id: colorMark
            visible: false
            action: markerAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        Menu { // Menu de la toolbar
            id: menuMark
            MenuItem { 
                text: isMarkerFull()? qsTr('All markers are in use !') : qsTr('Add a marker here')
                enabled: !isMarkerFull() && !isMarker()
                onTriggered: {
                    if (m1.text == '0') {
                        m1.text = timeline.position
                    } 
                    else if (m2.text == '0') {
                        m2.text = timeline.position
                    } 
                    else if (m3.text == '0') {
                        m3.text = timeline.position
                    } 
                    else if (m4.text == '0') {
                        m4.text = timeline.position
                    } 
                    else if (m5.text == '0') {
                        m5.text = timeline.position
                    }
                }
            }
            MenuItem { 
                text: qsTr('Delete this marker')
                enabled: isMarker() && timeline.position >= 1
                onTriggered: {
                    if (timeline.position == m1.text) {
                        if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
                        m1.text = '0'
                        choixcolor1.checked = false
                    }
                    if (timeline.position == m2.text) {
                        if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
                        m2.text = '0'
                        choixcolor2.checked = false
                    }
                    if (timeline.position == m3.text) {
                        if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
                        m3.text = '0'
                        choixcolor3.checked = false
                    }
                    if (timeline.position == m4.text) {
                        if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
                        m4.text = '0'
                        choixcolor4.checked = false
                    }
                    if (timeline.position == m5.text) {
                        if (timeline.position == origine.text) {selectedmark = 0 ; origine.text = '0'}
                        m5.text = '0'
                        choixcolor5.checked = false
                    }
                    return
                }
            }
            MenuSeparator {}
            Menu {
                title: qsTr('Move Playhead to Marker')
                MenuItem {
                    text: qsTr('Origin (0:00:00:00)')
                    onTriggered: timeline.position = 0
                }
                MenuItem {
                    text: (m1.text !== '0')? qsTr('Blue') + ' (' + application.timecode(m1.text).substr(1, 10) + ')' : qsTr('Blue')
                    enabled: m1.text !== '0'
                    onTriggered: timeline.position = m1.text
                }
                MenuItem {
                    text: (m2.text !== '0')? qsTr('Yellow') + ' (' + application.timecode(m2.text).substr(1, 10) + ')' : qsTr('Yellow')
                    enabled: m2.text !== '0'
                    onTriggered: timeline.position = m2.text
                }
                MenuItem {
                    text: (m3.text !== '0')? qsTr('Magenta') + ' (' + application.timecode(m3.text).substr(1, 10) + ')' : qsTr('Magenta')
                    enabled: m3.text !== '0'
                    onTriggered: timeline.position = m3.text
                }
                MenuItem {
                    text: (m4.text !== '0')? qsTr('Orange') + ' (' + application.timecode(m4.text).substr(1, 10) + ')' : qsTr('Orange')
                    enabled: m4.text !== '0'
                    onTriggered: timeline.position = m4.text
                }
                MenuItem {
                    text: (m5.text !== '0')? qsTr('Cyan') + ' (' + application.timecode(m5.text).substr(1, 10) + ')' : qsTr('Cyan')
                    enabled: m5.text !== '0'
                    onTriggered: timeline.position = m5.text
                }
            }
            MenuSeparator {}
            MenuItem { 
                text: qsTr('Delete all markers')
                onTriggered: {
                    selectedmark = 0
                    origine.text = m1.text = m2.text = m3.text = m4.text = m5.text = '0'
                    choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: aff_compteur
                text: qsTr('Display the timer')
                checkable: true
                checked: false
            }
            MenuSeparator {}
            MenuItem {
                text: qsTr('Cancel')
            }
        }
        Rectangle { // Compteur
            id: timer
            visible: aff_compteur.checked && colorMark.visible
            implicitWidth: 85
            implicitHeight: toolbar.height - 7
            color: '#FF1E1E1E'
            border.color: {
                (selectedmark == 1)? '#FF0090FF' : 
                (selectedmark == 2)? '#FFFFFF00' : 
                (selectedmark == 3)? '#FFFF00FF' : 
                (selectedmark == 4)? '#FFFF6600' : 
                (selectedmark == 5)? '#FF66FFFF' :
                '#FF000000'
            }
            border.width: 1
            Text {
                text: (timeline.position - origine.text >= 0)? " " + application.timecode(timeline.position - origine.text).substr(1, 10) : "-" + application.timecode(origine.text - timeline.position).substr(1, 10)
                font.family: 'Verdana'
                font.pointSize: (application.OS === 'OS X')? 11 : 8
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: 4
                color: '#FFE2E2E2'
            }
            MouseArea {
                id: mouseAreaTimer
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton
                onClicked: {
                    menuTimer.popup()
                }
            }
            ToolTip {
                visible: !mouseAreaTimer.containsMouse
                text: qsTr('Click to choose a reference marker')
            }
        }
        Menu { // Menu de la zone compteur
            id: menuTimer
            MenuItem { 
                text: qsTr('Choose a reference marker')
                enabled: false
            }
            MenuItem { 
                id: choixcolor0
                text: qsTr('Origin (0:00:00:00)')
                checkable: true
                checked: origine.text == '0'
                onTriggered: {
                    selectedmark = 0
                    origine.text = '0'
                    checked = true
                    choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor1
                text: (m1.text !== '0')? qsTr('Blue') + ' (' + application.timecode(m1.text).substr(1, 10) + ')' : qsTr('Blue')
                enabled: m1.text !== '0'
                checkable: true
                checked: origine.text == m1.text && m1.text !== '0'
                onTriggered: {
                    selectedmark = 1
                    origine.text = m1.text
                    checked = true
                    choixcolor0.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor2
                text: (m2.text !== '0')? qsTr('Yellow') + ' (' + application.timecode(m2.text).substr(1, 10) + ')' : qsTr('Yellow')
                enabled: m2.text !== '0'
                checkable: true
                checked: origine.text == m2.text && m2.text !== '0'
                onTriggered: {
                    selectedmark = 2
                    origine.text = m2.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor3.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor3
                text: (m3.text !== '0')? qsTr('Magenta') + ' (' + application.timecode(m3.text).substr(1, 10) + ')' : qsTr('Magenta')
                enabled: m3.text !== '0'
                checkable: true
                checked: origine.text == m3.text && m3.text !== '0'
                onTriggered: {
                    selectedmark = 3
                    origine.text = m3.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor4.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor4
                text: (m4.text !== '0')? qsTr('Orange') + ' (' + application.timecode(m4.text).substr(1, 10) + ')' : qsTr('Orange')
                enabled: m4.text !== '0'
                checkable: true
                checked: origine.text == m4.text && m4.text !== '0'
                onTriggered: {
                    selectedmark = 4
                    origine.text = m4.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor5.checked = false
                }
            }
            MenuItem { 
                id: choixcolor5
                text: (m5.text !== '0')? qsTr('Cyan') + ' (' + application.timecode(m5.text).substr(1, 10) + ')' : qsTr('Cyan')
                enabled: m5.text !== '0'
                checkable: true
                checked: origine.text == m5.text && m5.text !== '0'
                onTriggered: {
                    selectedmark = 5
                    origine.text = m5.text
                    checked = true
                    choixcolor0.checked = choixcolor1.checked = choixcolor2.checked = choixcolor3.checked = choixcolor4.checked = false
                }
            }
            MenuSeparator {}
            MenuItem {
                text: qsTr('Cancel')
            }
        }
        // ********* Fin Marqueurs et compteur partiel **************
        Button { // separator
            enabled: false
            implicitWidth: 5
            implicitHeight: 18
        }
        ToolButton {
            action: zoomOutAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ZoomSlider {
            id: scaleSlider
            implicitWidth: 180
            onValueChanged: scrollIfNeeded()
        }
        ToolButton {
            action: zoomInAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
        ToolButton {
            action: zoomFitAction
            implicitWidth: settings.smallIcons? 18 : hiddenButton.implicitWidth
            implicitHeight: implicitWidth
        }
    }

    Action {
        id: menuAction
        tooltip: qsTr('Display a menu of additional actions')
        iconName: 'show-menu'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/show-menu.png'
        onTriggered: menu.popup()
    }

    Action {
        id: cutAction
        tooltip: qsTr('Cut - Copy the current clip to the Source\nplayer and ripple delete it')
        iconName: 'edit-cut'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/edit-cut.png'
        enabled: timeline.selection.length
        onTriggered: timeline.removeSelection(true)
    }

    Action {
        id: copyAction
        tooltip: qsTr('Copy - Copy the current clip to the Source player (C)')
        iconName: 'edit-copy'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/edit-copy.png'
        enabled: timeline.selection.length
        onTriggered: timeline.copyClip(timeline.selection[0].y, timeline.selection[0].x)
    }

    Action {
        id: insertAction
        tooltip: qsTr('Paste - Insert clip into the current track\nshifting following clips to the right (V)')
        iconName: 'edit-paste'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/edit-paste.png'
        onTriggered: timeline.insert(currentTrack)
    }

    Action {
        id: appendAction
        tooltip: qsTr('Append to the current track (A)')
        iconName: 'list-add'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/list-add.png'
        onTriggered: timeline.append(currentTrack)
    }

    Action {
        id: deleteAction
        tooltip: qsTr('Ripple Delete - Remove current clip\nshifting following clips to the left (X)')
        iconName: 'list-remove'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/list-remove.png'
        onTriggered: timeline.removeSelection()
   }

    Action {
        id: liftAction
        tooltip: qsTr('Lift - Remove current clip without\naffecting position of other clips (Z)')
        iconName: 'lift'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/lift.png'
        onTriggered: timeline.liftSelection()
    }

    Action {
        id: overwriteAction
        tooltip: qsTr('Overwrite clip onto the current track (B)')
        iconName: 'overwrite'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/overwrite.png'
        onTriggered: timeline.overwrite(currentTrack)
    }

    Action {
        id: splitAction
        tooltip: qsTr('Split At Playhead (S)')
        iconName: 'slice'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/slice.png'
        onTriggered: timeline.splitClip(currentTrack)
    }

    Action {
        id: markerAction
        tooltip: qsTr("Display the marker menu")
        iconName: 'bookmarks'
        iconSource: 'qrc:///icons/oxygen/32x32/status/bookmarks.png'
        onTriggered: {
            menuMark.popup()
        }
    }

    Action {
        id: zoomOutAction
        tooltip: qsTr("Zoom timeline out (-)")
        iconName: 'zoom-out'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/zoom-out.png'
        onTriggered: {
        	root.zoomOut()
        	scrollIfNeeded()
        }
    }

    Action {
        id: zoomInAction
        tooltip: qsTr("Zoom timeline in (+)")
        iconName: 'zoom-in'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/zoom-in.png'
        onTriggered: {
        	root.zoomIn()
        	scrollIfNeeded()
        }
    }

    Action {
        id: zoomFitAction
        tooltip: qsTr('Zoom timeline to fit (0)')
        iconName: 'zoom-fit-best'
        iconSource: 'qrc:///icons/oxygen/32x32/actions/zoom-fit-best.png'
        onTriggered: {
            root.zoomToFit()
        }
    }
}
